/*
 * Materials for shadowed and or bumped surfaces
 */

#include <common.fxh>


/************* DATA STRUCTS **************/

struct appdata {
    float3 Position	: POSITION;
    float4 UV		: TEXCOORD0;
    float4 Normal	: NORMAL;
};

struct vertexOutput {
    float4 HPosition	: POSITION;
    float4 TexCoord	: TEXCOORD0;
	float3 WorldNormal : TEXCOORD1;
	float3 WorldPos : TEXCOORD2;
	float3 WorldEyePos : TEXCOORD3;
	float3 LightVec : TEXCOORD4;
	float3 HPos2 : TEXCOORD5;
};


/*********** vertex shaders ******/


vertexOutput NoBumpVS(appdata IN) {
    
	vertexOutput OUT;

	OUT.WorldNormal = mul(WorldIT, IN.Normal).xyz;

    float4 Po = float4(IN.Position.xyz,1.0);
    float3 Pw = mul(World, Po).xyz;
    OUT.WorldPos = Pw;
    OUT.TexCoord = IN.UV;
	OUT.WorldEyePos = ViewInv._m03_m13_m23;
    OUT.HPosition = mul(WorldViewProj, Po);

	OUT.LightVec = (LightPos.xyz - Pw);
	OUT.HPos2 = mul(WorldViewProj, Po);

    return OUT;
}

/********* pixel shaders ********/

float4 NoBumpPS_4(vertexOutput IN, float4 wPos : WPOS) : COLOR {

	float4 colorMap = tex2D(colorSampler,IN.TexCoord.xy);

	#include <ps_nobump_code.fxh>
	
	float4 diffContrib = (ldn*LightColor);
	float4 specContrib = ((ldn * hdn) * LightColor);

#ifdef USESHADOWS
	#include <ps_vsdct_code.fxh>

	#include <ps_pcf4tapdither.fxh>
	//#include <ps_pcf16tap.fxh>
#endif

	#include <ps_result_code.fxh>

	float depth = IN.HPos2.z;
	

#ifdef USEDOF
#include <ps_dof_alpha.fxh>
#else
	result.a = 1.0;
#endif

	return result;
}

float4 NoBumpPS_4_BlurFix(vertexOutput IN, float4 wPos : WPOS) : COLOR {

	float4 colorMap = tex2D(colorSampler,IN.TexCoord.xy);

	#include <ps_nobump_code.fxh>
	
	float4 diffContrib = (ldn*LightColor);
	float4 specContrib = ((ldn * hdn) * LightColor);
	
#ifdef USESHADOWS
	#include <ps_vsdct_code.fxh>

	#include <ps_pcf4tapdither.fxh>
	//#include <ps_pcf16tap.fxh>
#endif

	float depth = IN.HPos2.z;
	

#ifdef USEDOF
#include <ps_result_code.fxh>
#include <ps_dof_alpha.fxh>
	#ifdef USESHADOWS
		#include <ps_vsdctBlurFix.fxh>
	#endif
#else
#include <ps_result_code.fxh>
	result.a = 1.0;
#endif

	return result;
}

technique PerPixel_Shadow4Tap {
	pass p0 {
#include <commonRenderStates.fxh>
		CullfaceEnable = true;
		CullFace = Back;
		DepthTestEnable = true;
		VertexProgram = compile arbvp1 NoBumpVS();
		FragmentProgram = compile fp40 NoBumpPS_4();
	}
}

technique PerPixel_Shadow4Tap_BlurFix {
	pass p0 {
		#include <commonRenderStates.fxh>
		CullfaceEnable = true;
		CullFace = Back;
		DepthTestEnable = true;
		VertexProgram = compile arbvp1 NoBumpVS();
		FragmentProgram = compile fp40 NoBumpPS_4_BlurFix();
	}
}

//
//
////DEBUG
//float4 texsimple(uniform sampler2D sampler,
//                float2 uv : TEXCOORD0) : COLOR {
//    return tex2D(sampler, uv);
//}
//
//technique TextureOnly {
//    pass {
//		//AlphaTestEnable = false;
//		CullFaceEnable = true;
//		DepthTestEnable = false;
//		//ColorMask =  bool4(false, false, false, false);
//        FragmentProgram = compile arbfp1 texsimple(colorSampler);
//    }
//}
